package PLex::Lexer;

use strict;
use PLex;
use vars qw($VERSION @ISA);

require AutoLoader;

@ISA = qw(PLex AutoLoader);
$VERSION = '0.01';

# Preloaded methods
sub new {
    my ($class, $src_name) = @_;
    my $self = PLex->new();
    bless $self, $class;

    $self->{'_lex_tree'} = [ ];
    $self->{'_line'} = 1;
    $self->{'_source'} = $src_name;
    return $self;
}

# AUTOLOAD stubs for inheritance
sub lex;
sub add_token;
sub tree;
sub line;
sub source;
sub dump;
sub _extract_block;

# 1;
# __END__

=head1 NAME

PLex::Lexer - PLex text tokenizer module

=head1 SYNOPSIS

  use PLex::Lexer;

=head1 DESCRIPTION

The lexer breaks ablock of text into tokens. Each token is a hash
with a type and a value, $token->{'type'} and $token->{'val'}, and
they are chained together in an array. For recursive structures, val
is another token array. For error-reporting, tokens also include 'src'
and 'line', the filename and line-number respectively where the token
is located.

The lex() function is undefined--you will have to derive a class from
this one to lex your language that defines a lex() method. 

=head1 METHODS

The PLex::Lexer object provides the following methods:

=over 4

=item *

new()

I<Usage>

    $aks = new PLex::Lexer($src);

I<Description>

Creates a new Lexer with line number 1 and source file name set to
$src.  The lexer actually does not know anything about loading files:
it deals strictly with strings.

=item *

lex()

I<Description>

This is an undefined stub. It should take a string as an argument and
return a reference to an array of tokens. It should return -1 on error
and use error() to set a descriptive error string.

Internally, you should use add_token() to add tokens to the lex tree.

=item *

add_token()

I<Usage>

    $lexer->add_token($type, $val);

I<Description>

Adds a new token to the lex tree. This automatically sets the source
and line number for you. Please use this function rather than directly
addressing the internal lex tree, as the format may change in the
future.

=item *

tree()

I<Usage>

    $lexer->tree();

I<Description>

returns the lex tree

=item *

switch()

I<Usage>

    $old_tree = $lexer->switch($tree)

I<Description>

forces the lexer to dump to a new token tree--to enable restoration,
it returns the old tree

=item *

line()

I<Usage>

    $lexer->line();

I<Description>

returns the current file line

=item *

source()

I<Usage>

    $lexer->source();

I<Description>

returns the current file source

=item *

dump()

I<Usage>

    $lexer->dump();

I<Description>

Lexer debugging tool: dumps a representation of the lex tree to
STDOUT. So long as you follow the lexer interface it should work for
any lex tree.

=item *

_extract_block()

I<Usage>

    $contained_str = $lexer->_extract_block($ldelim, $rdelim, \$text);

I<Description>

Utility function: takes the left and right delimiters for some structure
(parens for function arguments, brackets for array indices) and correctly
finds the contained string even when there are many recursive instances of the same element. It returns the full string contained.

=back

=head1 AUTHOR

Kyle Downey, kdowney@xline.com

=head1 COPYRIGHT

Copyright 1997, X-Line Software. This software is freely-redistributable
under the terms of the GNU General Public License

=head1 SEE ALSO

PLex(3).

=cut

sub add_token {
    my ($self, $type, $val) = @_;

    my $tree = $self->tree();
    push @$tree, { type => $type, val => $val, src => $self->source(),
		   line => $self->line() };
}

sub switch {
    my ($self, $tree) = @_;

    # start with an empty list of tokens
    if (!ref($tree)) {
	$tree = [ ];
    }

    my $old_tree = $self->tree();
    $self->{'_lex_tree'} = $tree;
    return $old_tree;
}

sub tree {
    return shift->{'_lex_tree'};
}

sub source {
    return shift->{'_source'};
}

sub line {
    return shift->{'_line'};
}

# this dumps a text representation of the tree to standard
# output
sub dump {
    my ($self, $tree, $level) = @_;

    my ($elem, $old_tree);
    while ($elem = shift @$tree) {
	print "[$elem->{'src'}:$elem->{'line'}]: ";
	print '-' x $level;
	if ($elem->{'type'} eq 'var') {
	    my $var_chain = $elem->{'val'};

	    my $var_part;
	    while ($var_part = shift @$var_chain) {
		if (ref($var_part->{'val'})) {
		    print "[[array]]\n";
		    $old_tree = $self->switch($var_part->{'val'});
		    $self->dump($level + 1);
		    $self->switch($old_tree);
		}
		else {
		    print $var_part->{'val'},"\n";
		}
	    }
	}
	elsif ($elem->{'type'} eq 'func') {
	    print "func: ",$elem->{'val'}->{'name'},"\n";
	    $old_tree = $self->switch($elem->{'val'}->{'args'});
	    $self->dump($level + 1);
	    $self->switch($old_tree);
	}
	else {
	    print $elem->{'type'}, ": ", $elem->{'val'};
	}
	print "\n";
    }
}

# a utility parsing function that generically finds the boundaries
# of things like ( ... ), { ... } and [ ... ]; useful for parsing
# recursive structures like functions within functions without getting
# bitten by finding end characters before the true end of the function
sub _extract_block {
    my ($left_delim, $right_delim, $script) = @_;

    $$script = $left_delim . $$script;
    
    # find the next delim
    my $lcursor = 1;
    my $rcursor = 1;
    my ($lb_pos, $rb_pos);
    while (1) {
	$lb_pos = index $$script, $left_delim, $lcursor;
	$rb_pos = index $$script, $right_delim, $rcursor;
	if (($lb_pos == -1) || ($rb_pos < $lb_pos)) {
	    last;
	}
	else {
	    $lcursor = $lb_pos + 1;
	    $rcursor = $rb_pos + 1;
	}
    }
    my $index_str = substr $$script, 1, $rb_pos - 1;
    $$script = substr $$script, $rb_pos + 1;
    
    return $index_str;
}

1;

__END__
