package PLex::Namespace;

use PLex;
use strict;
use vars qw($VERSION @ISA);

require AutoLoader;

@ISA = qw(PLex AutoLoader);
@ISA = qw(PLex);
$VERSION = '0.01';

# Preloaded methods
sub new {
    my ($class) = @_;
    my $self = PLex->new;

    bless $self, $class;
    return $self;
}

# AUTOLOAD stubs
sub import;
sub export;
sub defined;
sub create;
sub get;
sub set;
sub compartment;

# 1;
# __END__

=head1 NAME

PLex::Namespace - isolated namespaces for language variable entitities

=head1 SYNOPSIS

  use PLex::Namespace;

=head1 DESCRIPTION

A Namespace is a generic, compartmentalized space for storing program
state information. This can include variable values of different
types, custom function definitions, and more. Each different type can
be stored in its own "compartment."

The get(), set() and create() functions expect to find child-defined
programs of the form get_*(), set_*() and create_*(), where * is the
name of a given type/compartment. The implicit contract is that these
funcitons will use compartment() to return a reference to a
compartment in the namespace and will store their data there, whatever
form it takes, and not in some other part of the object. This is
required so import() and export() work properly. 

=head1 METHODS

The PLex::Namespace object provides the following methods:

=over 4

=item *

new()

I<Usage>

    $namespace = new PLex::Namespace;

I<Description>

Creates a new Namespace object with an empty namespace. 

=item *

import(), export()

I<Usage>

    $namespace->import($namespace2->export(), $clobber);

I<Description>

import() lets you add the contents of one Namespace to another.  If
the second argument is set to true, import() will not raise an
exception if there are any conflicts between the namespaces. The
default behavior is to die when this happens (the equivalent in C or
Perl would be a "shadowed" global variable in a local scope). You
should always only use export() as a source for import(), as this
guarantees compatibility (the internal structure of Namespaces should
not be relied upon).

=item *

defined()

I<Usage>

    $namespace->$defined("foo", "scalar");

I<Description>

return true if the given item is defined in the given namespace
compartment

=item *

create()

I<Usage>

    $namespace->create("$i", "scalar");

I<Description>

creates a slot in the given namespace compartment for the item.

=item *

get()

I<Usage>

    $namespace->get("$i", "scalar", \$res);

I<Description>

gets the named item from the given compartment

=item *

set()

I<Usage>

    $namespace->set("$i", "scalar", $val);

I<Description>

sets the named variable in the given compartment

=item *

compartment()

I<Usage>

    my $cmp = $self->compartment('name');

I<Description>

returns a reference to the named compartment.

=back

=head1 AUTHOR

Kyle Downey, kdowney@xline.com

=head1 COPYRIGHT

Copyright 1997, X-Line Software. This software is freely-redistributable
under the terms of the GNU General Public License

=head1 SEE ALSO

PLex(3), PLex::Lexer(3), PLex::Parser(3), PLex::Functions(3), PLex::Operators(3), PLex::Context(3).

=cut

# adds the contents of one namespace to another. If $clobber
# is true, it will overwrite variables in the same namespace
sub import {
    my ($self, $namespace, $clobber) = @_;

    my $var;
    foreach $var (keys %$namespace) {
	if (exists $self->{'_vars'}->{$namespace->{$var}}) {
	    if ($clobber) {
		$self->{'_vars'}->{$var} = $namespace->{$var};
	    }
	    else {
		return $self->error("attempted to overwrite existing variable without clobbering enabled when import()ing namespace");
	    }
	}
	else {
	    $self->{'_vars'}->{$var} = $namespace->{$var};
	}
    }
    return 1;
}

sub export {
    return shift->{'_vars'};
}

# type is either 'var' or 'func'
sub defined {
    my ($self, $name, $type) = @_;

    ($type =~ /^(scalar|array|junc|func)$/) || return $self->error("searching for $name with unknown type ($type)");
    return exists $self->{'_vars'}->{$type}->{$name};
}

sub get {
    my ($self, $var, $type, $ret_val) = @_;
    return eval("\$self->get_$type(\$var, \$ret_val)");
}

sub set {
    my ($self, $var, $type, $val) = @_;
    return eval("\$self->set_$type(\$var, \$val)");
}

sub delete {
    my ($self, $var, $type) = @_;
    delete $self->compartment($type)->{$var};
    return 1;
}

sub create {
    my ($self, $var, $type) = @_;
    return eval("\$self->create_$type(\$var)");
}

sub compartment {
    my ($self, $compartment) = @_;

    unless (exists $self->{'_vars'}->{$compartment}) {
	$self->{'_vars'}->{$compartment} = { };
    }
    return $self->{'_vars'}->{$compartment};
}

1;

__END__
